
--[[
    Future posibilities:  See ticket log

    Thanks to Kaelten (OneBag), pastamancer (Grid), Tuller (Combuctor) and ckknight (tons!)
        for their excellent, clean code to work from as reference points.
--]]

ZOOKEEPER_VERSION = "1.0.1"
TYPE_MOUNTS = "MOUNT"
TYPE_COMPANIONS = "CRITTER"

ZooKeeper = LibStub("AceAddon-3.0"):NewAddon("ZooKeeper", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0", "LibFuBarPlugin-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("ZooKeeper", true)

-- BINDINGS
BINDING_HEADER_ZOOKEEPER = L.BindingHeader
BINDING_NAME_ZOOKEEPER_RECALL_HOTKEY = L.RecallHotkey

function ZooKeeper:OnInitialize()
    -- persist db
    local baseDB = LibStub("AceDB-3.0"):New("ZooKeeperDB", nil, "Defaults")
    self.db = baseDB
    self:InitDB()

    -- options
    self:SetupOptions()

    -- settings for fubar plugin
    self.name = "ZooKeeper"
    self.category = 'Interface Enhancements'
    self:SetFuBarOption('iconPath', 'Interface\\Addons\\ZooKeeper\\zk_icon.tga')

    self:SetFuBarOption('aceConfig30', 'ZooKeeper')
    self:SetFuBarOption('configType', 'AceConfigDialog-3.0')
end

function ZooKeeper:InitDB()
    local version = self.db.char.version
    if not version or version ~= ZOOKEEPER_VERSION then
        self.db.char.version = ZOOKEEPER_VERSION
    end
    if self.db.global.version == 1.0 then
        -- in version 1.0, everything was global (bleck)
        -- copy from global to profile default or char
        self.db:SetProfile("Defaults")
        self.db.char.companionOrder = self.db.global.companionOrder
        self.db.char.lastCompanion = self.db.global.lastCompanion
        self.db.char.lastViewed = self.db.global.lastViewed
        self.db.char.mountOrder = self.db.global.mountOrder
        self.db.char.position = self.db.global.position
        self.db.char.searchFilter = self.db.global.searchFilter
        self.db.char.version = self.db.global.version
        self.db.profile.buttonPadding = self.db.global.buttonPadding
        self.db.profile.columns = self.db.global.columns
        self.db.profile.opacity = self.db.global.opacity
        self.db.profile.position = self.db.global.position
        self.db.profile.recallOnLanding = self.db.global.recallOnLanding
        self.db.profile.recallOnRez = self.db.global.recallOnRez
        self.db.profile.recallOnZone = self.db.global.recallOnZone
        self.db.profile.scale = self.db.global.scale
        self.db.profile.showWithPetPaperDoll = self.db.global.showWithPetPaperDoll
        self.db.global.version = nil
    end

    if not self.db.profile.columns then
        self.db.profile.columns = 6
    end
    if not self.db.profile.rows then
        self.db.profile.rows = 10
    end
    if not self.db.char.lastViewed then
        self.db.char.lastViewed = TYPE_MOUNTS
    end
    if not self.db.profile.buttonPadding then
        self.db.profile.buttonPadding = 4
    end
    if not self.db.profile.scale then
        self.db.profile.scale = 0.8
    end
    if not self.db.profile.opacity then
        self.db.profile.opacity = 0.7
    end
    if not self.db.char.mountOrder then
        self.db.char.mountOrder = {}
    end
    if not self.db.char.companionOrder then
        self.db.char.companionOrder = {}
    end
end


function ZooKeeper:OnEnable()
    -- Called when the addon is enabled
    self.frame = self.Frame:New(L.Title, self.db, 'pets');
    self.fuBarOptionsFrame = self.ZooFuFrame:New(L.Title, self.db);

    self.onTaxi = nil

    -- register stuffs
    self:RegisterEvent("COMPANION_UPDATE", "CompanionUpdateHandler");
    self:RegisterEvent("COMPANION_LEARNED", "CompanionLearnedHandler");
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "EnteringWorldHandler");
    self:RegisterEvent("PLAYER_UNGHOST", "UnghostHandler");
    self:ScheduleRepeatingTimer(self.OnUpdate, 2, self)

    -- create buttons
    if not self.button then
        self.button = CreateFrame("Button", "ZooKeeperOpenButton", PetPaperDollFrame, "ZooKeeperOpenButtonTemplate");
        self.button:ClearAllPoints();
        self.button:SetPoint("TOPRIGHT", PetPaperDollFrameCompanionFrame, "TOPRIGHT", -36, -36);
        self.button.frame = self.frame;
        self.button:SetScript("OnClick", function(self) self.frame:ToggleFrame() end);
        self:SecureHook("CharacterFrameTab_OnClick");
    end
end

function ZooKeeper:OnDisable()
    -- flight taxi catcher
    self.frame:SetScript("ON_UPDATE", nil);

    -- unregister
    self:UnregisterAllEvents();

    -- cancel timers
    self:CancelAllTimers();
end

function ZooKeeper:CompanionUpdateHandler()
    if self.frame then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:CompanionLearnedHandler()
    self:Update()
    local optionsFrame = self.fuBarOptionsFrame
    optionsFrame:CreateMenu()
end

function ZooKeeper:Update()
    if self.frame and self.frame:IsShown() then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:CharacterFrameTab_OnClick (charFrame, button)
    local name = charFrame:GetName();

    if ( name == "CharacterFrameTab2" ) then
        if ( not self.frame:IsShown() ) then
            if (self.db.profile.showWithPetPaperDoll and self.db.profile.showWithPetPaperDoll == true) then
                self:ShowZoo()
            end
        end
    end
end

function ZooKeeper:ShowZoo()
    self.frame:ShowFrame()
end

function ZooKeeper:ToggleZoo()
    if self.frame:IsShown() then
        self.frame:Hide()
    else
        self.frame:Show()
    end
end

function ZooKeeper:UnghostHandler()
    if self.db.profile.recallOnRez then
        self:RecallLastCompanion()
    end
end

-- Called on PLAYER_ENTERING_WORLD (anytime a load screen is shown)
function ZooKeeper:EnteringWorldHandler()
    if self.db.profile.recallOnZone then
        self:RecallLastCompanion()
    end
end

function ZooKeeper:RecallLastCompanion()
    local petID = self.db.char.lastCompanion
    if petID then
        local creatureID, creatureName, spellID, icon, active = GetCompanionInfo(TYPE_COMPANIONS, petID);
        if (not active) then
            CallCompanion(TYPE_COMPANIONS, petID)
         end
    end
end

-- Taxi catcher, should rewrite some other way but PLAYER_LOST_CONTROL and PLAYER_GAINED_CONTROL doesn't seem to work.
--    Could instead only set the OnUpdate timer if recallOnLanding flag is set
function ZooKeeper:OnUpdate()
    if self.db.profile.recallOnLanding then
        local newOnTaxi = UnitOnTaxi("player")
        if self.onTaxi and not newOnTaxi then
            -- flight ended
            self:RecallLastCompanion()
        end

        self.onTaxi = newOnTaxi
    end

    if self.db.profile.recallOnDismount then
        local newOnMount = IsMounted()
        if self.onMount and not newOnMount then
            self:RecallLastCompanion()
        end

        self.onMount = newOnMount
    end
end

--[[
    Fubar plugin
--]]

function ZooKeeper:OnFuBarClick()
    self:ToggleZoo()
end

function ZooKeeper:OnFuBarEnter()
    local fuBarFrame = self:GetFrame()
    self.fuBarOptionsFrame:ShowMenu(fuBarFrame)
end

function ZooKeeper:OnFuBarLeave()
    local optionsFrame = self.fuBarOptionsFrame
    optionsFrame:StartCounting(optionsFrame)
end

function ZooKeeper:OnUpdateFuBarText()
    self:SetFuBarText(L.ShortTitle)
    self:ShowFuBarIcon()
end

--utility function: create a widget class (from Combuctor)
function ZooKeeper:NewClass(type)
    local class = CreateFrame(type)
    class.mt = {__index = class}

    function class:Bind(o)
        return setmetatable(o, self.mt)
    end

    return class
end
